QUAVER ; -------------------------------------------------------
 ; QUAVER, A SHORT MUSIC COMPILER FOR THE TRS-80 MODEL I
 ; (C) 1981 DENNIS BATHORY KITSZ, ROXBURY, VERMONT 05669
 ; -------------------------------------------------------
 ; QUAVER IS WRITTEN AS A BASIC LISTING, EACH LINE WITH
 ; AN INITIAL "REM" STATEMENT (TO AVOID THE BASIC TOKEN
 ; CREATION PROCESS).  UNIQUE KEYWORDS OR ABBREVIATIONS
 ; ARE USED FOR COMPILER INSTRUCTIONS AND DATA.
 ; -------------------------------------------------------
 ; BELOW IS THE LIST OF KEYWORDS OR ABBREVIATIONS THAT ARE
 ; SUPPORTED IN THIS VERSION.  ALL KEYWORDS BEGIN WITH A
 ; PERIOD.  THE FIRST TWO LETTERS OF THE KEYWORD ONLY ARE
 ; SIGNIFICANT; THE REST ARE IGNORED BY THE COMPILER.  FOR
 ; USE WITH LEFT / RIGHT SPEAKERS, ABBREVIATIONS (.LT AND
 ; .RT) ARE REQUIRED.
 ; -------------------------------------------------------
 ; PRESTO (.PR), ALLEGRO (.AL), MODERATO (.MO), ANDANTE
 ;  (.AN), LARGO (.LA) -- COMPILER DEFAULT IS MODERATO.
 ; RALLENTANDO (.RA), ACCELERANDO (.AC) -- INCREASE OR
 ;  DECREASE CONTINUES UNTIL MAXIMUM VALUE OR ANOTHER
 ;  TEMPO INDICATION IS REACHED.
 ; FORTE (.FO), MEZZOFORTE (.MF), MEZZOPIANO (.MP) --
 ;  COMPILER DEFAULT IS MEZZOFORTE.
 ; LEFT (.LT), RIGHT (.RT) -- WITH TWO-CASSETTE SYSTEM
 ;  ONLY -- COMPILER DEFAULT IS LEFT (CASSETTE ZERO).
 ; OBOE (.OB), CLARINET (.CL), FLUTE (.FL), VIOLIN (.VI)
 ;  -- COMPILER DEFAULT IS FULL SQUARE WAVE (CLARINET).
 ;  THESE "TIMBRES" ARE FOR REFERENCE ONLY AND DO NOT
 ;  REPRESENT ACTUAL INSTRUMENTAL SOUNDS.
 ; CON TREMOLO (.CO.TR), SENZA TREMOLO (.SE.TR) --
 ;  COMPILER DEFAULT IS SENZA TREMOLO (NOT USED THIS VER.)
 ; CON SORDINO (.CO.SO), SENZA SORDINO (.SE.SO) --
 ;  COMPILER DEFAULT IS SENZA SORDINO (NOT USED THIS VER.)
 ; STACCATO (.ST), LEGATO (.LE) -- COMPILER DEFAULT IS
 ;  LEGATO.
 ; -------------------------------------------------------
 ; REPEAT IS MARKED <: AND :> AND MAY NOT BE NESTED.
 ; BARLINES ARE MARKED WITH POINTS (!) AND ARE IGNORED BY
 ;  THE COMPILER.  THEY ARE FOR VISUAL REFERENCE.
 ; THE DOUBLE BAR IS MARKED >> AND CONCLUDES COMPILATION
 ;  OF THE MAIN SCORE. REFRAINS MUST FOLLOW THE DOUBLE BAR
 ;  AND ARE COMPILED SEPARATELY.  THE DOUBLE BAR IS NEEDED
 ;  FOR COMPILATION TO TAKE PLACE.
 ; -------------------------------------------------------
 ; NOTES TO BE PLAYED ARE ENCLOSED IN PARENTHESES (   ).
 ; NOTES ARE BY LETTER NAME (A B C D E F G), ACCIDENTAL
 ;  USING + FOR SHARP, - FOR FLAT, AND = FOR NATURAL.  THE
 ;  DEFAULT IS TO NATURAL.  ACCIDENTALS MUST BE REPEATED.
 ; OCTAVES ARE NOTED BY CHANGE.  THERE ARE FOUR OCTAVES
 ;  (1 - 4) AND UP TO C IN OCTAVE 5. DEFAULT IS THE THIRD
 ;  OCTAVE, OR THE LAST OCTAVE SOUNDED.
 ; NOTE LENGTH CHANGE IS INDICATED BY AN @ SIGN AND THE 
 ;  NOTE LENGTH VALUE.  @1 = EIGHTH NOTE, @2 = QUARTER
 ;  NOTE, @4 = HALF NOTE, @8 = WHOLE NOTE.  VALUES @3,
 ;  @5, @6, @7 ARE VALID AND USEFUL IN TRIPLE RHYTHMS.
 ; DOTTED VALUES ARE INDICATED BY A PLUS SIGN (+), AND
 ;  DOUBLY-DOTTED VALUES BY TWO PLUS SIGNS (++), FOLLOWING
 ;  THE NOTE LENGTH VALUE.
 ; AN IDENTICAL NOTE MAY BE REPEATED BY USING A SLASH (/).
 ; IDENTICAL NOTES TIED TOGETHER ARE INDICATED BY AN
 ;  AMPERSAND (&) SEPARATING THEM.
 ; TREMOLO ON A GIVEN NOTE IS INDICATED BY A PRECEDING
 ;  LETTER T.
 ; RESTS ARE NOTED BY THE LETTER R IN THE NOTE SEQUENCE,
 ;  FOLLOWED BY ITS LENGTH AS WITH NOTE VALUES ABOVE.
 ;  DEFAULT TO THE PREVIOUS LENGTH IS ALLOWED.
 ; IRREGULAR NOTE VALUES (TRIPLETS, QUINTUPLETS, ETC.)
 ;  ARE NOT SUPPORTED IN THIS VERSION.
 ; FOR A REFRAIN (A MUSICAL SUBROUTINE), THE MAIN
 ;  MUSICAL STREAM INCLUDES A # SIGN FOLLOWED BY A
 ;  REFRAIN LABEL.  THE REFRAIN IS INDICATED BY A SEGNO
 ;  ($) AND COMPLETED BY AN INVERTED VERSO ([).
 ;  THE REFRAIN PROCESSOR IS NOT SUPPORTED THIS VERSION.
 ; REMARKS BEGIN WITH AN ASTERISK (*) AND ARE IGNORED BY
 ;  THE COMPILER.  THEY MUST APPEAR OUTSIDE PARENTHESES.
 ; COMPILER COMMANDS INCLUDE BASIC FUNCTIONS PLUS:
 ;  "COMP", "PLAY", AND "DUMP".  ALL ROUTINES MUST BE
 ;  USED BEFORE ANY BASIC FUNCTIONS ARE INVOKED, OR THE
 ;  COMPILED ROUTINES WILL BE DESTROYED.
 ; SPACES ARE IGNORED BY THE COMPILER AND MAY BE USED
 ;  FOR CLARITY.
 ; THE SOURCE CODE MAY BE SAVED WITH THE BASIC "CSAVE"
 ;  FUNCTION, AS IT IS AN ORDINARY BASIC LISTING.
 ; -------------------------------------------------------
 ; ERROR MESSAGES:
 ; "QUAVER SYNTAX ERROR IN 00000" INDICATES GROSS ERROR IN
 ;  ENTRY WHICH CANNOT BE COMPILED.
 ; "RHYTHM VALUE ERROR IN 00000" INDICATES THAT AN
 ;  UNDEFINED RHYTHM VALUE (I.E., 9) HAS BEEN USED AND
 ;  CANNOT BE COMPILED.
 ; "INVALID NOTE OR NOTE OUT OF RANGE IN 00000" -- THE 
 ;  ONLY NOTES ALLOWED IN OCTAVE 5 ARE C= AND BELOW.
 ; "NESTED REPEAT IN 00000" -- THESE ARE NOT ALLOWED.
 ; "REPEAT WITHOUT START IN 00000" -- A FINAL REPEAT WAS
 ;  ENCOUNTERED WITHOUT AN OPENING REPEAT.
 ; "REPEAT NOT COMPLETED AT " -- A REPEAT WAS OPENED BUT
 ;  NOT CLOSED.  CLOSING OF THE REPEAT IS REQUIRED.
 ; "REFRAIN LABEL NOT FOUND IN 00000" -- NO REFRAIN LABEL
 ;  WAS DEFINED; COMPILATION CANNOT BE COMPLETED.
 ; "INVALID OR MISSPELLED KEYWORD IN 00000" -- NOTE THAT
 ;  .LT AND .RT CANNOT BE SPELLED OUT, AND THAT MODERATO
 ;  IS .MO AND FORTE IS .FO.
 ; "VERSO WITHOUT REFRAIN IN 00000" -- A RETURN TO VERSO
 ;  SYMBOL ([) WAS INSERTED IN A SECTION WHICH IS NOT OR
 ;  NOT YET A REFRAIN.  COMPILATION CANNOT BE COMPLETED.
 ; "OUT OF MEMORY IN 00000" -- IF THE COMPILATION WAS
 ;  NEARLY COMPLETE, SOME HAND COMPRESSION OF THE SOURCE
 ;  CODE WILL SOLVE THE PROBLEM.
 ; "LOGICAL ERROR IN 00000" -- THIS MIGHT BE AN IMPROPER
 ;  TIE, RHYTHMIC PERPLEXITY, REST WITH AN OCTAVE, ETC.
 ;  COMPILATION CANNOT BE COMPLETED.
 ; "NO DOUBLE BAR" -- A DOUBLE BAR IS NECESSARY, AND IS
 ;  PLACED BEFORE REFRAINS.
 ; -------------------------------------------------------
 ; 			SAMPLE LISTING
 ; -------------------------------------------------------
 ; .ALLEGRO .FORTE .LT .VIOLIN * DEFINE FIRST SOUNDS
 ; ! (E=3@8 / / C = 3@1 R@1 D=3@8 / / B=2@1 R@1) !
 ; .RT .CLARINET * CHANGE TO RIGHT SPEAKER; ALTER TIMBRE
 ; ! (E=3@8 / / C=3@1 R@1 D=3@8 / / B=2@1 R@1) 1
 ; #REFRAIN #REFRAIN * PLAY THE REFRAIN TWICE IN A ROW
 ; .LT .VIOLIN .MPIANO .ANDANTE * DEFINE NEW SOUNDS
 ; ! (E-2@4+ D=2@8 C=2@2 B-2@2++ A-2@8 G=G@1 & G=2@1) !
 ; .OBOE .MFORTE * AND DEFINE NEW TIMBRE AND VOLUME AGAIN
 ; ! (TC=4@1 TD=4@2 TE-4@2 D-4@1) !>>
 ; $REFRAIN ! (C+3@6 C+4@6 C+3@6 C+4@6 C+3@6 C=4@4) ! [
 ; -------------------------------------------------------
 ; 		COMPRESSED SAMPLE LISTING
 ; -------------------------------------------------------
 ; .AL.FO.LT.VI(E3@8//C@1RD@8//B2@1R).RT.CL(E3@8//C@1RD@8
 ; //B2@1R)#REFRAIN#REFRAIN.LT.VI.MP.AN(E-2@4+D@8C@2B-@2++
 ; A-@8G@1&).OB.MF(TC4@1TD@2TE-D-@1)>>$REFRAIN(C+3@6C+4C+3
 ; C+4C+3C4@4)[
 ; -------------------------------------------------------
 ; NOTICE THAT SPACES ARE REMOVED, REDUNDANT TIMING
 ; IS DELETED, AND REDUNDANT OCTAVES HAVE BEEN ELIMINATED.
 ; -------------------------------------------------------
 ; THIS COMPILER MAY BE EXPANDED WITH COMMANDS SUCH AS:
 ;	.AD	ANDANTINO
 ;	.AO	ALLEGRETTO
 ;	.CM	COMPRESS PITCHES
 ;	.EX	EXPAND PITCHES
 ;	.FF	FORTISSIMO
 ;	.IN	INVERT PITCHES
 ;	.MP	MEZZOPIANO
 ;	.PO	PORTAMENTO
 ;	.PT	PRESTISSIMO
 ;	.PP	PIANISSIMO
 ; -------------------------------------------------------
 ;
 ;
 ; USE OF REGISTERS:
 ;	IX POINTS TO MEMORY AREA FOR NOTES AND RHYTHMS
 ;	HL POINTS TO SOURCE CODE DURING COMPILATION
 ;	DE POINTS TO SUBROUTINE CALLS DURING COMPILATION
 ;	BC IS THE GENERAL WORKING REGISTER
 ;	HL POINTS TO THE NOTE LENGTH DURING PLAYBACK
 ;	BC POINTS TO THE WAVEFORM VALUES DURING PLAYBACK
 ;	IX POINTS TO THE PITCHES/RHYTHMS DURING PLAYBACK
 ;	ALTERNATE REGISTERS ARE NOT USED IN THE COMPILER
 ;	IY REGISTER IS NOT USED IN THE COMPILER
 ;
 ; GENERAL INFORMATION:
 ; COMPILED CODE BEGINS AT (40F9H), THE END OF BASIC
 ; 	SOURCE CODE AS USED BY BASIC PROGRAMS.
 ; SOURCE CODE FOR BASIC IS TAKEN FROM (40A9H).
 ;
 ; -------------------------------------------------------
 ;
